#' ---
#' title: "Preprocess data"
#' author: Timo Gnambs
#' date: "`r Sys.time()`"
#' output:
#'    html_document:
#'        toc: true
#'        number_sections: true
#'        toc_float: true
#'        theme: readable
#'        highlight: tango
#'        df_print: paged
#'        code_folding: show
#' ---


# Clear work space
rm(list = ls())

# Load packages
suppressPackageStartupMessages({
    library(tidyverse)
    library(haven)
    library(doBy)
    library(jsonlite)
    library(openxlsx)
})
source(here::here("Code/Helper.R"))

# Set options
Sys.setlocale("LC_TIME", "English")



#'
#' # Load rawdata
#'
#' The data from the **European Value Survey** (ESS) were retrieved
#' from https://europeansocialsurvey.org. Relevant information
#' is included in the *integrated file* and
#' *data from Interviewer's questionnaire*.
#'



#'
#' ## ESS round 1
#'
#' ESS Round 1: European Social Survey Round 1 Data (2002).
#' Data file edition 6.6. NSD - Norwegian Centre for Research
#' Data, Norway – Data Archive and distributor of ESS data for
#' ESS ERIC. [doi:10.21338/NSD-ESS1-2002](https://doi.org/10.21338/NSD-ESS1-2002).
#'
#' Round 1 does not provide an interviewer identification number or
#' information on the interviewers' sex and age. Therefore, the interviewer
#' identification number is imputed from the contact form using the
#' interviewer making the first contact. For missing information, the
#' respective identification number of the second or third interviewer is
#' imputed.
#'

path <- "~/Datasets/ESS/Round 1/"
dat1   <- read_spss(paste0(path, "ESS1e06_6.sav"))    # integrated data
dat2   <- read_spss(paste0(path, "ess1contacts.sav")) # contact forms
rm(path)

ess1 <- left_join(dat1  %>%
                      rename(sex = gndr,
                             age = agea,
                             work = mnactic,
                             hours = wkhct,
                             mig = ctzcntr,
                             relig = rlgdgr,
                             children = chldhm,
                             ls = stflife,
                             wgt = pspwght,
                             inwdds = inwdd,
                             inwmms = inwmm,
                             inwyys = inwyr) %>%
                      select(idno, cntry, sex, age, eduyrs, work, hours,
                             mig, relig, children, partner,
                             ipcrtiv:impfun, ls, happy, wgt,
                             inwdds, inwmms, inwyys, inwshh, inwsmm),
                  dat2 %>%
                      filter(!is.na(IDNO)) %>%
                      rename(idno = IDNO,
                             cntry = CNTRY) %>%
                      mutate(intnum = ifelse(is.na(INTNUM1), INTNUM2, INTNUM1),
                             intnum = ifelse(is.na(intnum), INTNUM2, intnum)) %>%
                      select(idno, cntry, intnum),
                  by = c("idno", "cntry")) %>%
        rename(cntryISO = cntry) %>%
        mutate(wave = 1,
               isex = NA,
               iage = NA)
rm(dat1, dat2)



#'
#' ## ESS round 2
#'
#' ESS Round 2: European Social Survey Round 2 Data (2004).
#' Data file edition 3.6. NSD - Norwegian Centre for Research
#' Data, Norway – Data Archive and distributor of ESS data for
#' ESS ERIC. [doi:10.21338/NSD-ESS2-2004](https:://10.21338/NSD-ESS2-2004).
#'
#' Round 2 does not provide information on the interviewers' sex and age.
#'
#' There are inconsistencies for the Icelandic respondent identification
#' number. Because the interviewer data could not be properly matched
#' for all respondents, the interviewer identification number is
#' missing for about 40% of the Icelandic data.
#'

path <- "~/Datasets/ESS/Round 2/"
dat1   <- read_spss(paste0(path, "ESS2e03_6.sav"))    # integrated data
dat1IT <- read_spss(paste0(path, "ESS2IT.sav"))       # data for Italy
dat2   <- read_spss(paste0(path, "ESS2INTe03_2.por")) # interviewer questionnaire
rm(path)

ess2 <- left_join(bind_rows(dat1, dat1IT) %>%
                      rename(sex = gndr,
                             age = agea,
                             work = mnactic,
                             hours = wkhct,
                             mig = ctzcntr,
                             relig = rlgdgr,
                             children = chldhm,
                             ls = stflife,
                             wgt = pspwght,
                             inwdds = inwdd,
                             inwmms = inwmm,
                             inwyys = inwyr) %>%
                      select(idno, cntry, sex, age, eduyrs, work, hours,
                             mig, relig, children, partner,
                             ipcrtiv:impfun, ls, happy, wgt,
                             inwdds, inwmms, inwyys, inwshh, inwsmm),
                  dat2 %>%
                      rename(idno = IDNO,
                             cntry = CNTRY,
                             intnum = INTNUM) %>%
                      select(idno, cntry, intnum),
                  by = c("idno", "cntry")) %>%
        mutate(wave = 2,
               isex = NA,
               iage = NA) %>%
        rename(cntryISO = cntry)
rm(dat1, dat1IT, dat2)



#'
#' ## ESS round 3
#'
#' ESS Round 3: European Social Survey Round 3 Data (2006).
#' Data file edition 3.7. NSD - Norwegian Centre for Research
#' Data, Norway – Data Archive and distributor of ESS data for
#' ESS ERIC. [doi:10.21338/NSD-ESS3-2006](https::/doi.org/10.21338/NSD-ESS3-2006).
#'
#' Round 3 does not provide information on the interviewers' sex and age.
#'
#' No design weights are available for Latvia and Romania.
#'

path <- "~/Datasets/ESS/Round 3/"
dat1     <- read_spss(paste0(path, "ESS3e03_7.sav"))  # integrated data
dat1LVRO <- read_spss(paste0(path, "ESS3LVRO.sav"))   # data for Latvia and Romania
dat2     <- read_spss(paste0(path, "ESS3INTe02.por")) # interviewer questionnaire
rm(path)

ess3 <- left_join(bind_rows(dat1, dat1LVRO) %>%
                        select(-age) %>%
                        rename(sex = gndr,
                               age = agea,
                               work = mnactic,
                               hours = wkhct,
                               mig = ctzcntr,
                               relig = rlgdgr,
                               children = chldhm,
                               ls = stflife,
                               wgt = pspwght) %>%
                        select(idno, cntry, sex, age, eduyrs, work, hours,
                               mig, relig, children, partner,
                               ipcrtiv:impfun, ls, happy, wgt,
                               inwdds, inwmms, inwyys, inwshh, inwsmm) %>%
                        # no design weights for Latvia and Romania
                        mutate(wgt = ifelse(is.na(wgt), 1, wgt)),
                  dat2 %>%
                        rename(idno = IDNO, cntry = CNTRY, intnum = INTNUM) %>%
                        select(idno, cntry, intnum),
                  by = c("idno", "cntry")) %>%
        mutate(wave = 3,
               isex = NA,
               iage = NA) %>%
        rename(cntryISO = cntry)
rm(dat1, dat1LVRO, dat2)



#'
#' ## ESS round 4
#'
#' ESS Round 4: European Social Survey Round 4 Data (2008).
#' Data file edition 4.5. NSD - Norwegian Centre for Research
#' Data, Norway – Data Archive and distributor of ESS data for
#' ESS ERIC. [doi:10.21338/NSD-ESS4-2008](https://doi.org/10.21338/NSD-ESS4-2008).
#'
#' Information on the interviewers' sex and age is not provided for Sweden.
#'
#' No design weights are available for Latvia.
#'

path <- "~/Datasets/ESS/Round 4/"
dat1   <- read_spss(paste0(path, "ESS4e04_5.sav"))  # integrated data
dat1AT <- read_spss(paste0(path, "ESS4AT.sav"))     # data for Austria
dat1LT <- read_spss(paste0(path, "ESS4LT.sav"))     # data for Latvia
dat2   <- read_spss(paste0(path, "ESS4INTe03.sav")) # interviewer questionnaire
rm(path)

ess4 <- left_join(bind_rows(dat1, dat1AT, dat1LT) %>%
                        rename(sex = gndr,
                               age = agea,
                               work = mnactic,
                               hours = wkhct,
                               mig = ctzcntr,
                               relig = rlgdgr,
                               children = chldhm,
                               ls = stflife,
                               wgt = pspwght) %>%
                        select(idno, cntry, sex, age, eduyrs, work, hours,
                               mig, relig, children, partner,
                               ipcrtiv:impfun, ls, happy, wgt,
                               inwdds, inwmms, inwyys, inwshh, inwsmm) %>%
                        # no design weights for Latvia
                        mutate(wgt = ifelse(is.na(wgt), 1, wgt),
                               wave = 4),
                    dat2 %>%
                        rename(isex = intgndr,
                               iage = intage) %>%
                        select(idno, cntry, intnum, iage, isex),
                  by = c("idno", "cntry")) %>%
        rename(cntryISO = cntry)
rm(dat1, dat1AT, dat1LT, dat2)



#'
#' ## ESS round 5
#'
#' ESS Round 5: European Social Survey Round 5 Data (2010).
#' Data file edition 3.4. NSD - Norwegian Centre for Research
#' Data, Norway – Data Archive and distributor of ESS data for
#' ESS ERIC. [doi:10.21338/NSD-ESS5-2010](https://doi.org/10.21338/NSD-ESS5-2010).
#'
#' Information on the interviewers' sex and age is not provided for Sweden.
#'
#' Information on the interview date is not available for Estonia.
#'
#' Information on the interview time is not available for Ireland
#'

path <- "~/Datasets/ESS/Round 5/"
dat1   <- read_spss(paste0(path, "ESS5e03_4.sav"))   # integrated data
dat1AT <- read_spss(paste0(path, "ESS5ATe1_1.sav"))  # data for Austria
dat2   <- read_spss(paste0(path, "ESS5INTe03.sav"))  # interviewer questionnaire
dat2AT <- read_spss(paste0(path, "ESS5intAT.sav"))   # interviewer questionnaire for Austria
rm(path)

ess5 <- left_join(bind_rows(dat1, dat1AT) %>%
                        rename(sex = gndr,
                               age = agea,
                               work = mnactic,
                               hours = wkhct,
                               mig = ctzcntr,
                               relig = rlgdgr,
                               children = chldhm,
                               partner = icpart1,
                               ls = stflife,
                               wgt = pspwght) %>%
                        select(idno, cntry, sex, age, eduyrs, work, hours,
                               mig, relig, children, partner,
                               ipcrtiv:impfun, ls, happy, wgt,
                               inwdds, inwmms, inwyys, inwshh, inwsmm) %>%
                        mutate(wave = 5),
                  bind_rows(dat2, dat2AT) %>%
                        rename(isex = intgndr,
                               iage = intage) %>%
                        # create age categories
                        mutate(intagea = cut(intagea, c(0, 30, 40, 50, 60, 70, 100)),
                               intagea = as.numeric(intagea),
                               iage = ifelse(is.na(iage), intagea, iage)) %>%
                        select(idno, cntry, intnum, iage, isex),
                  by = c("idno", "cntry")) %>%
        rename(cntryISO = cntry)
rm(dat1, dat1AT, dat2, dat2AT)

#'
#' For 7 interviewers in Cyprus, inconsistent information on their sex
#' and age is provided. These inconsistencies are corrected.
#'

ess5 <- left_join(ess5 %>%
                      select(-isex, -iage),
                  ess5 %>%
                      group_by(cntryISO, intnum) %>%
                      summarise(isex = Mode(isex),
                                iage = Mode(iage),
                                .groups = "keep"),
                  by = c("cntryISO", "intnum"))



#'
#' ## ESS round 6
#'
#' ESS Round 6: European Social Survey Round 6 Data (2012).
#' Data file edition 2.4. NSD - Norwegian Centre for Research
#' Data, Norway – Data Archive and distributor of ESS data for
#' ESS ERIC. [doi:10.21338/NSD-ESS6-2012](https://doi.org/10.21338/NSD-ESS6-2012).
#'
#' Information on the interviewers' sex and age is not provided for Ukraine.
#'

path <- "~/Datasets/ESS/Round 6/"
dat1 <- read_spss(paste0(path, "ESS6e02_4.sav"))  # integrated data
dat2 <- read_spss(paste0(path, "ESS6INTe02_1.sav")) # interviewer questionnaire
rm(path)

ess6 <- left_join(dat1 %>%
                        rename(sex = gndr,
                               age = agea,
                               work = mnactic,
                               hours = wkhct,
                               mig = ctzcntr,
                               relig = rlgdgr,
                               children = chldhm,
                               partner = icpart1,
                               ls = stflife,
                               wgt = pspwght) %>%
                        select(idno, cntry, sex, age, eduyrs, work, hours,
                               mig, relig, children, partner,
                               ipcrtiv:impfun, ls, happy, wgt,
                               inwdds, inwmms, inwyys, inwshh, inwsmm) %>%
                        mutate(wave = 6),
                  dat2 %>%
                        rename(isex = intgndr,
                               iage = intagea) %>%
                        # create age categories
                        mutate(iage = cut(iage, c(0, 30, 40, 50, 60, 70, 100)),
                               iage = as.numeric(iage)) %>%
                        select(idno, cntry, intnum, iage, isex),
                  by = c("idno", "cntry")) %>%
        rename(cntryISO = cntry)
rm(dat1, dat2)

#'
#' For 2 interviewers in the United Kingdom, inconsistent information on their sex
#' and age is provided. These inconsistencies are corrected.
#'

ess6 <- left_join(ess6 %>%
                      select(-isex, -iage),
                  ess6 %>%
                      group_by(cntryISO, intnum) %>%
                      summarise(isex = Mode(isex),
                                iage = Mode(iage),
                                .groups = "keep"),
                  by = c("cntryISO", "intnum"))



#'
#' ## ESS round 7
#'
#' ESS Round 7: European Social Survey Round 7 Data (2014).
#' Data file edition 2.2. NSD - Norwegian Centre for Research
#' Data, Norway – Data Archive and distributor of ESS data for
#' ESS ERIC. [doi:10.21338/NSD-ESS7-2014](https://doi.org/10.21338/NSD-ESS7-2014).
#'
#' Information on the interview time is not available for Portugal.
#'

path <- "~/Datasets/ESS/Round 7/"
dat1 <- read_spss(paste0(path, "ESS7e02_2.sav"))  # integrated data
dat2 <- read_spss(paste0(path, "ESS7INTe02_1.sav")) # interviewer questionnaire
rm(path)

ess7 <- left_join(dat1 %>%
                        rename(sex = gndr,
                               age = agea,
                               work = mnactic,
                               hours = wkhct,
                               mig = ctzcntr,
                               relig = rlgdgr,
                               children = chldhm,
                               partner = icpart1,
                               ls = stflife,
                               wgt = pspwght) %>%
                        select(idno, cntry, sex, age, eduyrs, work, hours,
                               mig, relig, children, partner,
                               ipcrtiv:impfun, ls, happy, wgt,
                               inwdds, inwmms, inwyys, inwshh, inwsmm) %>%
                        mutate(wave = 7),
                  dat2 %>%
                        rename(isex = intgndr,
                               iage = intagea) %>%
                        # create age categories
                        mutate(iage = cut(iage, c(0, 30, 40, 50, 60, 70, 100)),
                               iage = as.numeric(iage)) %>%
                        select(idno, cntry, intnum, iage, isex),
                  by = c("idno", "cntry")) %>%
        rename(cntryISO = cntry)
rm(dat1, dat2)



#'
#' ## ESS round 8
#'
#' ESS Round 8: European Social Survey Round 8 Data (2016).
#' Data file edition 2.1. NSD - Norwegian Centre for Research
#' Data, Norway – Data Archive and distributor of ESS data for
#' ESS ERIC. [doi:10.21338/NSD-ESS8-2016](https://doi.org/10.21338/NSD-ESS8-2016).
#'

path <- "~/Datasets/ESS/Round 8/"
dat1 <- read_spss(paste0(path, "ESS8e02_1.sav"))  # integrated data
dat2 <- read_spss(paste0(path, "ESS8INTe02.sav")) # interviewer questionnaire
rm(path)

ess8 <- left_join(dat1 %>%
                        rename(sex = gndr,
                               age = agea,
                               work = mnactic,
                               hours = wkhct,
                               mig = ctzcntr,
                               relig = rlgdgr,
                               children = chldhm,
                               partner = icpart1,
                               ls = stflife,
                               wgt = pspwght) %>%
                        select(idno, cntry, sex, age, eduyrs, work, hours,
                               mig, relig, children, partner,
                               ipcrtiv:impfun, ls, happy, wgt,
                               inwdds, inwmms, inwyys, inwshh, inwsmm) %>%
                        mutate(wave = 8),
                  dat2 %>%
                        rename(isex = intgndr,
                               iage = intagea) %>%
                        # create age categories
                        mutate(iage = cut(iage, c(0, 30, 40, 50, 60, 70, 100)),
                               iage = as.numeric(iage)) %>%
                        select(idno, cntry, intnum, iage, isex),
                  by = c("idno", "cntry")) %>%
        rename(cntryISO = cntry)
rm(dat1, dat2)



#'
#' ## ESS round 9
#'
#'
#' ESS Round 9: European Social Survey Round 9 Data (2018).
#' Data file edition 2.0. NSD - Norwegian Centre for Research
#' Data, Norway – Data Archive and distributor of ESS data for
#' ESS ERIC. [doi:10.21338/NSD-ESS9-2018](https://doi.org/10.21338/NSD-ESS9-2018).
#'
#' Information on the interviewers' age is not provided
#' for Montenegro.
#'
#' Information on the interview date is not available for Bulgaria.
#'

path <- "~/Datasets/ESS/Round 9/"
dat1 <- read_spss(paste0(path, "ESS9e02.sav"))    # integrated data
dat2 <- read_spss(paste0(path, "ESS9INTe02.sav")) # interviewer questionnaire
rm(path)

ess9 <- left_join(dat1 %>%
                        mutate(across(rshipa2:rshipa10, ~{.x %in% 1}, .names = "X{.col}")) %>%
                        mutate(across(rshipa2:rshipa10, ~{.x %in% 2}, .names = "Y{.col}")) %>%
                        rowwise() %>%
                        mutate(partner = ifelse(sum(Xrshipa2:Xrshipa10) > 0, 1, 2),
                               children = ifelse(sum(Yrshipa2:Yrshipa10) > 0, 1, 2)) %>%
                        ungroup() %>%
                        rename(sex = gndr,
                               age = agea,
                               work = mnactic,
                               hours = wkhct,
                               mig = ctzcntr,
                               relig = rlgdgr,
                               mig = ctzcntr,
                               ls = stflife,
                               wgt = pspwght) %>%
                        select(idno, cntry, sex, age, eduyrs, work, hours,
                               mig, relig, children, partner,
                               ipcrtiv:impfun, ls, happy, wgt,
                               inwdds, inwmms, inwyys, inwshh, inwsmm) %>%
                        mutate(wave = 9),
                  dat2 %>%
                        rename(isex = intgndr,
                               iage = intagea) %>%
                        # create age categories
                        mutate(iage = cut(iage, c(0, 30, 40, 50, 60, 70, 100)),
                               iage = as.numeric(iage)) %>%
                        select(idno, cntry, intnum, iage, isex),
                  by = c("idno", "cntry")) %>%
        rename(cntryISO = cntry)
rm(dat1, dat2)

#'
#' For 2 interviewers in Montenegro, inconsistent information on their sex
#' and age is provided. These inconsistencies are corrected.
#'

ess9 <- left_join(ess9 %>%
                      select(-isex, -iage),
                  ess9 %>%
                      group_by(cntryISO, intnum) %>%
                      summarise(isex = Mode(isex),
                                iage = Mode(iage),
                                .groups = "keep"),
                  by = c("cntryISO", "intnum"))



#'
#' ## Merge ESS rounds
#'

dat <- bind_rows(ess1, ess2)
dat <- bind_rows(dat, ess3, ess4, ess5)
dat <- bind_rows(dat, ess6, ess7, ess8, ess9)
rm(ess1, ess2, ess3, ess4, ess5, ess6, ess7, ess8, ess9)



#'
#' # Recode variables
#'

# Country
attributes(dat$cntryISO) <- NULL
dat$cntry <- factor(dat$cntryISO,
                    levels = c("AL", "AT", "BE", "BG", "CH", "CY", "CZ",
                               "DE", "DK", "EE", "ES", "FI", "FR", "GB",
                               "GR", "HR", "HU", "IE", "IL", "IS", "IT",
                               "LT", "LU", "LV", "ME", "NL", "NO", "PL",
                               "PT", "RO", "RS", "RU", "SE", "SI", "SK",
                               "TR", "UA", "XK"),
                    labels = c("Albania", "Austria", "Belgium",
                               "Bulgaria", "Switzerland", "Cyprus",
                               "Czechia", "Germany", "Denmark",
                               "Estonia", "Spain", "Finland",
                               "France", "United Kingdom", "Greece",
                               "Croatia", "Hungary", "Ireland",
                               "Israel", "Iceland", "Italy",
                               "Lithuania", "Luxembourg", "Latvia",
                               "Montenegro", "Netherlands", "Norway",
                               "Poland", "Portugal", "Romania",
                               "Serbia", "Russia", "Sweden",
                               "Slovenia", "Slovakia", "Turkey",
                               "Ukraine", "Kosovo"))
dat$cntry <- relevel(dat$cntry, ref = "Germany")
table(dat$cntry, useNA = "always")

# Sex (1 = male, 2 = female)
dat$sex <- factor(dat$sex,
                  levels = c(1, 2),
                  labels = c("male", "female"))
dat$sex <- relevel(dat$sex, ref = "male")
table(dat$sex, useNA = "always")

# Age (in years)
dat$age[dat$age > 100] <- NA
table(dat$age, useNA = "always")

# Interviewer sex (1 = male, 2 = female)
# Note: Missing for several samples!
dat$isex <- factor(dat$isex,
                   levels = c(1, 2),
                   labels = c("male", "female"))
table(dat$isex, useNA = "always")

# Interviewer age (in categories)
# Note: Missing for several samples!
dat$iage <- factor(dat$iage,
                   levels = 1:6,
                   labels = c("-30", "31-40", "41-50",
                              "51-60", "61-70", "71+"))
table(dat$iage, useNA = "always")

# Years of full-time education completed
dat$eduyrs <- recodeVar(dat$eduyrs, 0:30, 0:30, default = NA)
table(dat$eduyrs, useNA = "always")

# Main activity last 7 days
# (1 = Paid work,
#  2 = Education,
#  3 = Unemployed, looking for job,
#  4 = Unemployed, not looking for job,
#  5 = Permanently sick or disabled,
#  6 = Retired,
#  7 = Community or military service,
#  8 = Housework, looking after children, others,
#  9 = Other)
dat$work <- recodeVar(dat$work,
                      1:9,
                      c(1, 2, 4, 4, 4, 3, 4, 4, 4),
                      default = NA)
dat$work <- factor(dat$work,
                   levels = 1:4,
                   labels = c("Paid work", "Education", "Retired", "Other"))
dat$work <- relevel(dat$work, ref = "Paid work")
table(dat$work, useNA = "always")

# Contractual work hours
dat$hours <- round(dat$hours)
dat$hours[dat$hours > 80] <- 80
dat$hours[dat$work != "Paid work"] <- 0
table(dat$hours, useNA = "always")

# Citizen of country (reverse coded as migration)
dat$mig <- factor(dat$mig, levels = 1:2, labels = c("No", "Yes"))
dat$mig <- relevel(dat$mig, ref = "No")
table(dat$mig, useNA = "always")

# Living with a partner (1 = yes, 2 = no)
dat$partner <- factor(dat$partner,
                      levels = c(1, 2),
                      labels = c("yes", "no"))
dat$partner <- relevel(dat$partner, ref = "yes")
table(dat$partner, useNA = "always")

# Living with children at home (1 = yes, 2 = no)
dat$children <- factor(dat$children,
                       levels = c(1, 2),
                       labels = c("yes", "no"))
dat$children <- relevel(dat$children, ref = "yes")
table(dat$children, useNA = "always")

# How religious are you
table(dat$relig, useNA = "always")

# Schwartz higher-order values
sapply(select(dat, ipcrtiv:impfun), table, useNA = "always")
dat <- dat %>%
          mutate(OTC = 7 - rowMeans(select(., "impdiff", "ipadvnt", "ipcrtiv",
                                              "impfree", "impfun", "ipgdtim"), na.rm = TRUE),
                 CON = 7 - rowMeans(select(., "ipmodst", "imptrad", "ipbhprp",
                                              "ipfrule", "impsafe", "ipstrgv"), na.rm = TRUE),
                 STR = 7 - rowMeans(select(., "iphlppl", "iplylfr", "ipeqopt",
                                              "ipudrst", "impenv"), na.rm = TRUE),
                 SEN = 7 - rowMeans(select(., "imprich", "iprspot", "ipshabt",
                                              "ipsuces"), na.rm = TRUE),
                 opendim = OTC - CON,
                 selfdim = STR - SEN)

# How satisfied with life as a whole
# (0 = Extremely dissatisfied,
#  10 = Extremely satisfied)
table(dat$ls, useNA = "always")

# How happy are you
# (0 = Extremely unhappy,
#  10 = Extremely happy)
table(dat$happy, useNA = "always")

# Country identification number
dat$cid <- factor(dat$cntryISO)
length(unique(dat$cid)) # number of countries

# Sample identification number
# Note: As wave and country to create a unique
#       identifier for each sample
dat$sid <- factor(paste0(dat$cntryISO, dat$wave))
length(unique(dat$sid)) # number of samples

# Interviewer identification number
# Note: Merged with sample identification number
#       to create a unique identifier across samples
dat$iid <- factor(paste0(dat$sid, "_", dat$intnum))
dat$iid[is.na(dat$intnum)] <- NA
median(c(table(dat$iid))) # median number of interviews per interviewer

# Day of the week
dat$date <- strptime(paste0(dat$inwdds, "/", dat$inwmms, "/", dat$inwyys),
                     "%d/%m/%Y")
dat$dow <- factor(weekdays(dat$date, abbreviate = FALSE),
                  levels = c("Monday", "Tuesday", "Wednesday",
                             "Thursday", "Friday", "Saturday", "Sunday"),
                  labels = c("Monday", "Tuesday", "Wednesday",
                             "Thursday", "Friday", "Saturday", "Sunday"))
table(dat$dow, useNA = "always")

# Dummy indicators for day of the week
dat$isMo <- factor(ifelse(dat$dow == "Monday", 1, 0),
                   levels = c(0, 1),
                   labels = c("other day", "Monday"))
dat$isMo <- relevel(dat$isMo, ref = "other day")
dat$isFr <- factor(ifelse(dat$dow == "Friday", 1, 0),
                   levels = c(0, 1),
                   labels = c("other day", "Friday"))
dat$isFr <- relevel(dat$isFr, ref = "other day")
dat$isSa <- factor(ifelse(dat$dow == "Saturday", 1, 0),
                   levels = c(0, 1),
                   labels = c("other day", "Saturday"))
dat$isSa <- relevel(dat$isSa, ref = "other day")
dat$isSu <- factor(ifelse(dat$dow == "Sunday", 1, 0),
                   levels = c(0, 1),
                   labels = c("other day", "Sunday"))
dat$isSu <- relevel(dat$isSu, ref = "other day")
table(dat$isMo, useNA = "always")
table(dat$isFr, useNA = "always")
table(dat$isSa, useNA = "always")
table(dat$isSu, useNA = "always")

# Quarter of the year
dat$quart <- recodeVar(dat$inwmms, 1:12, c(rep(1, 3), rep(2, 3),
                                           rep(3, 3), rep(4, 3)),
                       default = NA)
dat$quart <- factor(dat$quart,
                    levels = 1:4,
                    labels = c("1st", "2nd", "3rd", "4th"))
dat$quart <- relevel(dat$quart, ref = "1st")
table(dat$quart, useNA = "always")

# Time of the day
# Note: Missing for two samples.
dat$tod <- round(dat$inwshh) + round(dat$inwsmm) / 60
sum(dat$tod >= 22 | dat$tod <= 4, na.rm=T) / sum(!is.na(dat$tod)) # % recoded
dat$tod[dat$tod >= 22 | dat$tod <= 4] <- 22
dat$tod[dat$tod < 8] <- 8
table(round(dat$tod), useNA = "always")

# Number of interviews on specific days
#   conducted by each interviewer
dat <- left_join(dat,
                 dat %>%
                   group_by(iid) %>%
                   summarise(iisMo = sum(dow %in% "Monday"),
                             iisFr = sum(dow %in% "Friday"),
                             iisSa = sum(dow %in% "Saturday"),
                             iisSu = sum(dow %in% "Sunday"),
                             .groups = "keep") %>%
                   ungroup() %>%
                   mutate(iisMo = ifelse(is.na(iid), NA, iisMo),
                          iisFr = ifelse(is.na(iid), NA, iisFr),
                          iisSa = ifelse(is.na(iid), NA, iisSa),
                          iisSu = ifelse(is.na(iid), NA, iisSu)),
                 by = "iid")
table(dat$iisMo, useNA = "always")
table(dat$iisFr, useNA = "always")
table(dat$iisSa, useNA = "always")
table(dat$iisSu, useNA = "always")



#'
#' # Holidays
#'
#' Interviews conducted on a public holiday should be
#' excluded from the analyses. Therefore, all public holidays
#' for each sample are retrieved from https://date.nager.at.

#' Because the website does not list respective dates for
#' Kosovo, Israel, and Mazedonia, the respective public holidays
#' were manually retrieved from https://www.timeanddate.com/holidays.
#'

# Load holidays
if (!file.exists(here::here("Data/holidays.rds"))) {

    holidays <- c()

    # Retrieve public holidays from nager.at
    for (i in unique(dat$sid)) {

        # Country and survey year
        cntry <- unclass(dat$cntryISO[dat$sid == i])[1]
        year  <- unclass(dat$inwyys[dat$sid == i])[1]

        # Countries are not supported
        if (cntry %in% c("IL", "ME", "XK")) next

        # No year available
        if (is.na(year)) next

        # URL for API
        url <- paste0("https://date.nager.at/api/v2/publicholidays/",
                      year, "/", cntry)

        # Fetch data
        h <- fromJSON(url) %>%
                filter(type == "Public") %>%
                rename(cntryISO = countryCode, holiday_date = date,
                       holiday_name = name) %>%
                select(cntryISO, holiday_date, holiday_name)

        # Add data
        holidays <- bind_rows(holidays, h)
    }
    rm(i, cntry, year, url, h)

    # Retrieve manually coded public holidays from timeanddate.com
    h <- read.xlsx(here::here("Data/holidays_ILXKME.xlsx"),
                   sheet = "Holidays") %>%
            rename(holiday_name = name) %>%
            mutate(holiday_date = paste(year,
                                        str_pad(month, 2, "left", 0),
                                        str_pad(day, 2, "left", 0),
                                        sep = "-")) %>%
            select(cntryISO, holiday_name, holiday_date)
    holidays <- bind_rows(holidays, h) %>%
                    arrange(cntryISO, holiday_date)
    rm(h)

    # Save holidays
    saveRDS(holidays, here::here("Data/holidays.rds"))
}
holidays <- readRDS(here::here("Data/holidays.rds"))

# Determine holidays
dat$hday <- paste(dat$inwyys,
                  str_pad(dat$inwmms, 2, "left", 0),
                  str_pad(dat$inwdds, 2, "left", 0),
                  sep = "-")
dat$isHoliday <- apply(dat[, c("cntryISO", "hday")], 1, function(x) {
    as.numeric(x[2] %in% holidays$holiday_date[holidays$cntryISO %in% x[1]])
})
dat$isHoliday[is.na(dat$dow)] <- NA
dat$hday <- NULL
dat$isHoliday <- factor(dat$isHoliday,
                        levels = c(0, 1),
                        labels = c("no", "yes"))
table(dat$isHoliday, useNA = "always")
rm(holidays)



#'
#' # Select data
#'
#' ## Select cases
#'

# Available data
(N <- nrow(dat))               # sample size
(k <- length(unique(dat$sid))) # samples

# Samples without interview date
dat <- dat %>%
            filter(!(cntryISO %in% "EE" & wave == 5) &
                   !(cntryISO %in% "BG" & wave == 9))
dat %>%
    summarise('N' = n(),
              'k' = length(unique(sid)),
              'N of original' = paste0(round(N / !!N * 100, 2), "%"),
              'k of original' = paste0(round(k / !!k * 100, 2), "%"))

# Samples without interviewer identification number
dat <- dat %>%
            filter(!(cntryISO %in% c("AT", "FR", "SE") & wave == 1))
dat %>%
    summarise('N' = n(),
              'k' = length(unique(sid)),
              'N of original' = paste0(round(N / !!N * 100, 2), "%"),
              'k of original' = paste0(round(k / !!k * 100, 2), "%"))

# Samples from Israel
dat <- dat %>%
            filter(!(cntryISO %in% c("IL")))
dat %>%
    summarise('N' = n(),
              'k' = length(unique(sid)),
              'N of original' = paste0(round(N / !!N * 100, 2), "%"),
              'k of original' = paste0(round(k / !!k * 100, 2), "%"))

# Participants without interview day
dat <- dat %>%
          filter(!is.na(dow))
dat %>%
    summarise('N' = n(),
              'k' = length(unique(sid)),
              'N of original' = paste0(round(N / !!N * 100, 2), "%"),
              'k of original' = paste0(round(k / !!k * 100, 2), "%"))

# Interviews not on a public holiday
dat <- dat %>%
          filter(isHoliday == "no")
dat %>%
    summarise('N' = n(),
              'k' = length(unique(sid)),
              'N of original' = paste0(round(N / !!N * 100, 2), "%"),
              'k of original' = paste0(round(k / !!k * 100, 2), "%"))

# No interviewer identification number available
dat <- dat %>%
          filter(!is.na(iid))
dat %>%
    summarise('N' = n(),
              'k' = length(unique(sid)),
              'N of original' = paste0(round(N / !!N * 100, 2), "%"),
              'k of original' = paste0(round(k / !!k * 100, 2), "%"))
rm(N, k)



#'
#' ## Select variables
#'

dat <- dat %>%
         select(sid, cid, cntryISO, cntry, wave, iid, wgt,
                sex, age, eduyrs, work, hours,
                mig, relig, children, partner,
                opendim, selfdim, ls, happy,
                dow, tod, quart, isMo, isFr, isSa, isSu,
                isex, iage, iisMo, iisFr, iisSa, iisSu)



#'
#' ## Missing values
#'

# Missing values by item (%)
dat %>%
    select(-sid, -cntryISO, -cntry, -wave, -wgt) %>%
    summarise(across(everything(), ~mean(is.na(.x)))) %>%
    mutate(across(everything(), ~round(.x * 100, 2))) %>%
    pivot_longer(everything())

# Impute missing values (with < 5%) with mode/median
dat <- dat %>%
          mutate(across(c(sex, age, work, mig,
                          children, partner),
                        ~if_else(is.na(.), Mode(.), .))) %>%
          mutate(across(c(eduyrs, hours, relig, opendim,
                          selfdim, ls, happy, tod),
                        ~ifelse(is.na(.), median(., na.rm = TRUE), .)))

# Drop empty levels from factors
dat$cntry <- droplevels(dat$cntry)
dat$iid   <- droplevels(dat$iid)
dat$sid   <- droplevels(dat$sid)



#'
#' # Set contrasts
#'
#' For the day of the week indicators effect-coding
#' is used to examine the deviation from the mean,
#' while dummy-coding is used for the remaining
#' categorical variables.
#'

# Sex
contrasts(dat$sex) <- contr.treatment(2, base = 1)
colnames(attributes(dat$sex)$contrasts) <- "female"

# Work
contrasts(dat$work) <- contr.treatment(4, base = 1)
colnames(attributes(dat$work)$contrasts) <- levels(dat$work)[-1]

# Children at home
contrasts(dat$children) <- contr.treatment(2, base = 2)
colnames(attributes(dat$children)$contrasts) <- "yes"

# Living with partner
contrasts(dat$partner) <- contr.treatment(2, base = 2)
colnames(attributes(dat$partner)$contrasts) <- "yes"

# Migration
contrasts(dat$mig) <- contr.treatment(2, base = 1)
colnames(attributes(dat$mig)$contrasts) <- "yes"

# Quarter of year
contrasts(dat$quart) <- contr.treatment(4, base = 1)
colnames(attributes(dat$quart)$contrasts) <- levels(dat$quart)[-1]

# Indicator for Monday
contrasts(dat$isMo) <- rbind(-1, 1)
colnames(attributes(dat$isMo)$contrasts) <- "Monday"

# Indicator for Friday
contrasts(dat$isFr) <- rbind(-1, 1)
colnames(attributes(dat$isFr)$contrasts) <- "Friday"

# Indicator for Saturday
contrasts(dat$isSa) <- rbind(-1, 1)
colnames(attributes(dat$isSa)$contrasts) <- "Saturday"

# Indicator for Sunday
contrasts(dat$isSu) <- rbind(-1, 1)
colnames(attributes(dat$isSu)$contrasts) <- "Sunday"

# Sample identification number
contrasts(dat$sid) <- contr.sum(length(unique(dat$sid)))



#'
#' # Save data
#'

saveRDS(dat, here::here("Data/data.rds"))


