#' ---
#' title: "Selectivity analyses"
#' author: Timo Gnambs
#' date: "`r Sys.time()`"
#' output:
#'    html_document:
#'        toc: true
#'        number_sections: true
#'        toc_float: true
#'        theme: readable
#'        highlight: tango
#'        df_print: default
#'        code_folding: show
#' ---


# Clear work space
rm(list = ls())

# Load packages
suppressPackageStartupMessages({
    library(tidyverse)
    library(sandwich)
    library(lme4)
    library(lmtest)
})

# Load preprocessed data
dat <- readRDS(here::here("Data/data.rds"))



#'
#' # Monday versus remaining week
#'

# Selection model
if (file.exists(here::here("Data/smod1.rds"))) {
    smod1 <- readRDS(here::here("Data/smod1.rds"))
} else {
    smod1 <- glm(isMo ~ sex +
                        I(age / 10) +    # in 10-years units
                        eduyrs +
                        work +
                        hours +
                        partner +
                        children +
                        I(scale(relig)) +
                        I(scale(opendim)) +
                        I(scale(selfdim)) +
                        mig +
                        tod +
                        quart +
                        iisMo +
                        sid,
                 data = dat,
                 weights = wgt,
                 family = quasibinomial(link = "probit"))
    saveRDS(smod1, file = here::here("Data/smod1.rds"))
}
vars <- c("(Intercept)", "sexfemale", "I(age/10)",
          "eduyrs", "workEducation", "workRetired",
          "workOther", "hours", "partneryes",
          "childrenyes", "I(scale(relig))", "I(scale(opendim))",
          "I(scale(selfdim))", "migyes", "tod",
          "quart2nd", "quart3rd", "quart4th")
coeftest(smod1, vcov = vcovCL(smod1, dat$iid))[c(vars, "iisMo"), ]

# Inverse Mills ratio
dat$mrMo <- NA
dat$mrMo <- dnorm(smod1$linear.predictors) /
                pnorm(smod1$linear.predictors)



#'
#' # Friday versus remaining week
#'

# Selection model
if (file.exists(here::here("Data/smod2.rds"))) {
    smod2 <- readRDS(here::here("Data/smod2.rds"))
} else {
    smod2 <- update(smod1, isFr ~ . -iisMo + iisFr)
    saveRDS(smod2, file = here::here("Data/smod2.rds"))
}
coeftest(smod2, vcov = vcovCL(smod2, dat$iid))[c(vars, "iisFr"), ]

# Inverse Mills ratio
dat$mrFr <- NA
dat$mrFr <- dnorm(smod2$linear.predictors) /
                pnorm(smod2$linear.predictors)
rm(smod2)



#'
#' # Saturday versus remaining week
#'

# Selection model
if (file.exists(here::here("Data/smod3.rds"))) {
    smod3 <- readRDS(here::here("Data/smod3.rds"))
} else {
    smod3 <- update(smod1, isSa ~ . -iisMo + iisSa)
    saveRDS(smod3, file = here::here("Data/smod3.rds"))
}
coeftest(smod3, vcov = vcovCL(smod3, dat$iid))[c(vars, "iisSa"), ]

# Inverse Mills ratio
dat$mrSa <- NA
dat$mrSa <- dnorm(smod3$linear.predictors) /
                pnorm(smod3$linear.predictors)
rm(smod3)



#'
#' # Sunday versus remaining week
#'

# Selection model
if (file.exists(here::here("Data/smod4.rds"))) {
    smod4 <- readRDS(here::here("Data/smod4.rds"))
} else {
    smod4 <- update(smod1, isSu ~ . -iisMo + iisSu)
    saveRDS(smod4, file = here::here("Data/smod4.rds"))
}
coeftest(smod4, vcov = vcovCL(smod4, dat$iid))[c(vars, "iisSu"), ]

# Inverse Mills ratio
dat$mrSu <- NA
dat$mrSu <- dnorm(smod4$linear.predictors) /
                pnorm(smod4$linear.predictors)
rm(smod1, smod4, vars)



#'
#' # Save results
#'

saveRDS(dat, here::here("Data/data_imr.rds"))

