#' ---
#' title: "Meta-analyses"
#' author: Timo Gnambs
#' date: "`r Sys.time()`"
#' output:
#'    html_document:
#'        toc: true
#'        number_sections: true
#'        toc_float: true
#'        theme: readable
#'        highlight: tango
#'        df_print: paged
#'        code_folding: show
#' ---


# Clear work space
rm(list = ls())

# Load packages
suppressPackageStartupMessages({
    library(tidyverse)
    library(lme4)
    library(lmerTest)
    library(effectsize)
})
source(here::here("Code/Helper.R"))

# Load preprocessed data with invMR
dat <- readRDS(here::here("Data/data_imr.rds"))

# Recode day indicators to numeric
dat <- dat %>%
          mutate(isMo2 = ifelse(isMo == "Monday",   1, -1),
                 isFr2 = ifelse(isFr == "Friday",   1, -1),
                 isSa2 = ifelse(isSa == "Saturday", 1, -1),
                 isSu2 = ifelse(isSu == "Sunday",   1, -1))

# Standardize outcomes
dat$ls <- scale(dat$ls)
dat$happy <- scale(dat$happy)

# Round time of day to full hours
dat$tod_rnd <- round(dat$tod)



#'
#' # Intraclass Correlations
#'

# ICC for life satisfaction
if (file.exists(here::here("Data/ls0.rds"))) {
    ls0 <- readRDS(here::here("Data/ls0.rds"))
} else {
    ls0 <- lmer(ls ~ (1 | cid) + (1 | sid) + (1 | iid) +
                     (1 | dow) + (1 | tod_rnd),
                data = dat, weights = wgt)
    saveRDS(ls0, here::here("Data/ls0.rds"))
}
ICC(ls0, 6)

# ICC for happiness
if (file.exists(here::here("Data/hp0.rds"))) {
    hp0 <- readRDS(here::here("Data/hp0.rds"))
} else {
    hp0 <- lmer(happy ~ (1 | cid) + (1 | sid) + (1 | iid) +
                        (1 | dow) + (1 | tod_rnd),
                data = dat, weights = wgt)
    saveRDS(hp0, here::here("Data/hp0.rds"))
}
ICC(hp0, 6)



#'
#' # Meta-analyses for life satisfaction
#'
#' ## Fixed effects model
#'

if (file.exists(here::here("Data/ls1.rds"))) {
    ls1 <- readRDS(here::here("Data/ls1.rds"))
} else {
    ls1 <- lm(ls ~ isMo + isFr + isSa + isSu, data = dat, weights = wgt)
    saveRDS(ls1, here::here("Data/ls1.rds"))
}
summary(ls1)



#'
#' ## Random effects model
#'
#' Some of the more complex models with random slopes
#' result in convergence warnings regarding too large
#' scaled gradients at the fitted estimates. These seem
#' to be false positives because of the large sample
#' size (see ?convergence). Refitting these models using
#' the current parameter estimates as starting values,
#' gets rid of the warnings, suggesting a false positive
#' (see ?troubleshooting).
#'

# Without random slopes
if (file.exists(here::here("Data/ls2a.rds"))) {
    ls2a <- readRDS(here::here("Data/ls2a.rds"))
} else {
    ls2a <- lmer(ls ~ isMo + isFr + isSa + isSu +
                      (1 | cid) + (1 | sid) + (1 | iid),
                 data = dat, weights = wgt, REML = FALSE)
    saveRDS(ls2a, here::here("Data/ls2a.rds"))
}
print(summary(ls2a), correlation = FALSE)

# With random slopes for interviewers
if (file.exists(here::here("Data/ls2b.rds"))) {
    ls2b <- readRDS(here::here("Data/ls2b.rds"))
} else {
    ls2b <- lmer(ls ~ isMo + isFr + isSa + isSu +
                      (1 | cid) + (1 | sid) + (1 | iid) +
                      (0 + isMo2 + isFr2 + isSa2 + isSu2 || iid),
                 data = dat, weights = wgt, REML = FALSE,
                 control = lmerControl(optCtrl = list(ftol_abs = 1e-8,
                                                      xtol_abs = 1e-8,
                                                      maxeval = 3000)))
    ls2b <- update(ls2b, start = getME(ls2b, "theta"))
    saveRDS(ls2b, here::here("Data/ls2b.rds"))
}
print(summary(ls2b), correlation = FALSE)

# With random slopes for samples
if (file.exists(here::here("Data/ls2c.rds"))) {
    ls2c <- readRDS(here::here("Data/ls2c.rds"))
} else {
    ls2c <- lmer(ls ~ isMo + isFr + isSa + isSu +
                      (1 | cid) + (1 | sid) + (1 | iid) +
                      (0 + isMo2 + isFr2 + isSa2 + isSu2 || sid),
                 data = dat, weights = wgt, REML = FALSE,
                 control = lmerControl(optCtrl = list(ftol_abs = 1e-8,
                                                      xtol_abs = 1e-8,
                                                      maxeval = 3000)))
    ls2c <- update(ls2c, start = getME(ls2c, "theta"))
    saveRDS(ls2c, here::here("Data/ls2c.rds"))
}
print(summary(ls2c), correlation = FALSE)

# With random slopes for countries
if (file.exists(here::here("Data/ls2d.rds"))) {
    ls2d <- readRDS(here::here("Data/ls2d.rds"))
} else {
    ls2d <- lmer(ls ~ isMo + isFr + isSa + isSu +
                      (1 | cid) + (1 | sid) + (1 | iid) +
                      (0 + isMo2 + isFr2 + isSa2 + isSu2 || cid),
                 data = dat, weights = wgt, REML = FALSE,
                 control = lmerControl(optCtrl = list(ftol_abs = 1e-8,
                                                      xtol_abs = 1e-8,
                                                      maxeval = 3000)))
    ls2d <- update(ls2d, start = getME(ls2d, "theta"))
    saveRDS(ls2d, here::here("Data/ls2d.rds"))
}
print(summary(ls2d), correlation = FALSE)

# Model comparisons
tibble(Model = c("Fixed effects",
                 "Random intercept",
                 "Random slopes for interviewers",
                 "Random slopes for samples",
                 "Random slopes for countries"),
       BIC = c(BIC(ls1),  BIC(ls2a), BIC(ls2b),
               BIC(ls2c), BIC(ls2d)),
       AIC = c(AIC(ls1),  AIC(ls2a), AIC(ls2b),
               AIC(ls2c), AIC(ls2d)))
anova_(ls1, ls2a)
anova_(ls2a, ls2b)
anova_(ls2a, ls2c)
anova_(ls2a, ls2d)

# Heterogeneity
ICC(ls2a)
I2(ls2b, 4)



#'
#' ## Random effects model with selection variables
#'

if (file.exists(here::here("Data/ls3.rds"))) {
    ls3 <- readRDS(here::here("Data/ls3.rds"))
} else {
    ls3 <- update(ls2b, . ~ . + sex +
                                I(age / 10) +     # in 10-years units
                                eduyrs +
                                work +
                                hours +
                                partner +
                                children +
                                I(scale(relig)) +
                                I(scale(opendim)) +
                                I(scale(selfdim)) +
                                mig +
                                tod +
                                quart +
                                mrMo + mrFr + mrSa + mrSu)
    saveRDS(ls3, here::here("Data/ls3.rds"))
}
print(summary(ls3), correlation = FALSE)

# Heterogeneity
I2(ls3, 4)



#'
#' ## Moderating effects of work status
#'

# Moderating effects
if (file.exists(here::here("Data/ls4.rds"))) {
    ls4 <- readRDS(here::here("Data/ls4.rds"))
} else {
    ls4 <- update(ls3, . ~ . + work:isMo + work:isFr +
                               work:isSa + work:isSu)
    ls4 <- update(ls4, start = getME(ls4, "theta"))
    saveRDS(ls4, here::here("Data/ls4.rds"))
}
print(summary(ls4), correlation = FALSE)

# Model comparison
anova_(ls3, ls4)

# Effects for paid work
if (file.exists(here::here("Data/ls3a.rds"))) {
    ls3a <- readRDS(here::here("Data/ls3a.rds"))
} else {
    ls3a <- update(ls3, . ~ . - work,
                   subset = (dat$work == "Paid work"))
    saveRDS(ls3a, here::here("Data/ls3a.rds"))
}
print(summary(ls3a), correlation = FALSE)

# Effects for education
if (file.exists(here::here("Data/ls3b.rds"))) {
    ls3b <- readRDS(here::here("Data/ls3b.rds"))
} else {
    ls3b <- update(ls3, . ~ . - work - hours,
                   subset = (dat$work == "Education"))
    saveRDS(ls3b, here::here("Data/ls3b.rds"))
}
print(summary(ls3b), correlation = FALSE)

# Effects for retired
if (file.exists(here::here("Data/ls3c.rds"))) {
    ls3c <- readRDS(here::here("Data/ls3c.rds"))
} else {
    ls3c <- update(ls3, . ~ . - work - hours,
                   subset = (dat$work == "Retired"))
    saveRDS(ls3c, here::here("Data/ls3c.rds"))
}
print(summary(ls3c), correlation = FALSE)



#'
#' # Meta-analyses for happiness
#'
#' ## Fixed effects model
#'

if (file.exists(here::here("Data/hp1.rds"))) {
    hp1 <- readRDS(here::here("Data/hp1.rds"))
} else {
    hp1 <- lm(happy ~ isMo + isFr + isSa + isSu, data = dat, weights = wgt)
    saveRDS(hp1, here::here("Data/hp1.rds"))
}
print(summary(hp1), correlation = FALSE)



#'
#' ## Random effects model
#'
#' Some of the more complex models with random slopes
#' result in convergence warnings regarding too large
#' scaled gradients at the fitted estimates. These seem
#' to be false positives because of the large sample
#' size (see ?convergence). Refitting these models using
#' the current parameter estimates as starting values,
#' gets rid of the warnings, suggesting a false positive
#' (see ?troubleshooting).
#'

# Without random slopes
if (file.exists(here::here("Data/hp2a.rds"))) {
    hp2a <- readRDS(here::here("Data/hp2a.rds"))
} else {
    hp2a <- lmer(happy ~ isMo + isFr + isSa + isSu +
                         (1 | cid) + (1 | sid) + (1 | iid),
                 data = dat, weights = wgt, REML = FALSE)
    saveRDS(hp2a, here::here("Data/hp2a.rds"))
}
print(summary(hp2a), correlation = FALSE)

# With random slopes for interviewers
if (file.exists(here::here("Data/hp2b.rds"))) {
    hp2b <- readRDS(here::here("Data/hp2b.rds"))
} else {
    hp2b <- lmer(happy ~ isMo + isFr + isSa + isSu +
                         (1 | cid) + (1 | sid) + (1 | iid) +
                         (0 + isMo2 + isFr2 + isSa2 + isSu2 || iid),
                 data = dat, weights = wgt, REML = FALSE,
                 control = lmerControl(optCtrl = list(ftol_abs = 1e-8,
                                                      xtol_abs = 1e-8,
                                                      maxeval = 3000)))
    hp2b <- update(hp2b, start = getME(hp2b, "theta"))
    saveRDS(hp2b, here::here("Data/hp2b.rds"))
}
print(summary(hp2b), correlation = FALSE)

# With random slopes for samples
if (file.exists(here::here("Data/hp2c.rds"))) {
    hp2c <- readRDS(here::here("Data/hp2c.rds"))
} else {
    hp2c <- lmer(happy ~ isMo + isFr + isSa + isSu +
                         (1 | cid) + (1 | sid) + (1 | iid) +
                         (0 + isMo2 + isFr2 + isSa2 + isSu2 || sid),
                 data = dat, weights = wgt, REML = FALSE,
                 control = lmerControl(optCtrl = list(ftol_abs = 1e-8,
                                                      xtol_abs = 1e-8,
                                                      maxeval = 3000)))
    hp2c <- update(hp2c, start = getME(hp2c, c("theta", "fixef")))
    saveRDS(hp2c, here::here("Data/hp2c.rds"))
}
print(summary(hp2c), correlation = FALSE)

# With random slopes for countries
if (file.exists(here::here("Data/hp2d.rds"))) {
    hp2d <- readRDS(here::here("Data/hp2d.rds"))
} else {
    hp2d <- lmer(happy ~ isMo + isFr + isSa + isSu +
                         (1 | cid) + (1 | sid) + (1 | iid) +
                         (0 + isMo2 + isFr2 + isSa2 + isSu2 || cid),
                 data = dat, weights = wgt, REML = FALSE,
                 control = lmerControl(optCtrl = list(ftol_abs = 1e-8,
                                                      xtol_abs = 1e-8,
                                                      maxeval = 3000)))
    hp2d <- update(hp2d, start = getME(hp2d, c("theta", "fixef")))
    saveRDS(hp2d, here::here("Data/hp2d.rds"))
}
print(summary(hp2d), correlation = FALSE)

# Heterogeneity
ICC(hp2a)
I2(hp2b, 4)

# Model comparison
tibble(Model = c("Fixed effects",
                 "Random intercept",
                 "Random slopes for interviewers",
                 "Random slopes for samples",
                 "Random slopes for countries"),
       BIC = c(BIC(hp1),  BIC(hp2a), BIC(hp2b),
               BIC(hp2c), BIC(hp2d)),
       AIC = c(AIC(hp1),  AIC(hp2a), AIC(hp2b),
               AIC(hp2c), AIC(hp2d)))
anova_(hp1, hp2a)
anova_(hp2a, hp2b)
anova_(hp2a, hp2c)
anova_(hp2a, hp2d)



#'
#' ## Random effects model with selection variables
#'

if (file.exists(here::here("Data/hp3.rds"))) {
    hp3 <- readRDS(here::here("Data/hp3.rds"))
} else {
    hp3 <- update(hp2b, . ~ . + sex +
                                I(age / 10) +    # in 10-years units
                                eduyrs +
                                work +
                                hours +
                                partner +
                                children +
                                I(scale(relig)) +
                                I(scale(opendim)) +
                                I(scale(selfdim)) +
                                mig +
                                tod +
                                quart +
                                mrMo + mrFr + mrSa + mrSu)
    saveRDS(hp3, here::here("Data/hp3.rds"))
}
print(summary(hp3), correlation = FALSE)

# Heterogeneity
I2(hp3, 4)



#'
#' ## Moderating effects of work status
#'

# Moderating effects
if (file.exists(here::here("Data/hp4.rds"))) {
    hp4 <- readRDS(here::here("Data/hp4.rds"))
} else {
    hp4 <- update(hp3, . ~ . + work:isMo + work:isFr +
                               work:isSa + work:isSu)
    saveRDS(hp4, here::here("Data/hp4.rds"))
}
print(summary(hp4), correlation = FALSE)

# Model comparison
anova_(hp3, hp4)

# Effects for paid work
if (file.exists(here::here("Data/hp3a.rds"))) {
    hp3a <- readRDS(here::here("Data/hp3a.rds"))
} else {
    hp3a <- update(hp3, . ~ . - work,
                   subset = (dat$work == "Paid work"))
    saveRDS(hp3a, here::here("Data/hp3a.rds"))
}
print(summary(hp3a), correlation = FALSE)

# Effects for education
if (file.exists(here::here("Data/hp3b.rds"))) {
    hp3b <- readRDS(here::here("Data/hp3b.rds"))
} else {
    hp3b <- update(hp3, . ~ . - work - hours,
                   subset = (dat$work == "Education"))
    saveRDS(hp3b, here::here("Data/hp3b.rds"))
}
print(summary(hp3b), correlation = FALSE)

# Effects for retired
if (file.exists(here::here("Data/hp3c.rds"))) {
    hp3c <- readRDS(here::here("Data/hp3c.rds"))
} else {
    hp3c <- update(hp3, . ~ . - work - hours,
                   subset = (dat$work == "Retired"))
    saveRDS(hp3c, here::here("Data/hp3c.rds"))
}
print(summary(hp3c), correlation = FALSE)

