#' ---
#' title: "Descriptives"
#' author: Timo Gnambs
#' date: "`r Sys.time()`"
#' output:
#'    html_document:
#'        toc: true
#'        number_sections: true
#'        toc_float: true
#'        theme: readable
#'        highlight: tango
#'        df_print: paged
#'        code_folding: show
#' ---


# Clear work space
rm(list = ls())

# Load packages
suppressPackageStartupMessages({
    library(tidyverse)
    library(openxlsx)
    library(Hmisc)
})
source(here::here("Code/Helper.R"))

# Load preprocessed data with invMR
dat <- readRDS(here::here("Data/data_imr.rds"))

# Set options
options(pillar.sigfig = 4)



#'
#' # Participant characteristics
#'

dat %>%
    summarise('Samples (N)' = length(unique(sid)),
              'Interviewers (N)' = length(unique(iid)),
              'Countries (N)' = length(unique(cntryISO)),
              'Participants (N)' = n(),
              'Sample size (Mdn)' = median(table(sid)),
              'Sample size (Min)' = min(table(sid)),
              'Sample size (Max)' = max(table(sid)),
              'Female (N)' = sum(sex == "female"),
              'Female (%)' = round(mean(sex == "female") * 100, 2),
              "Age (M)" = mean(age, na.rm = TRUE),
              "Age (SD)" = sd(age, na.rm = TRUE),
              "Age (Min)" = min(age, na.rm = TRUE),
              "Age (Max)" = max(age, na.rm = TRUE),
              "Migration (%)" = round(mean(mig == "Yes") * 100, 2),
              "Years education (M)" = mean(eduyrs, na.rm = TRUE),
              "Years education (SD)" = sd(eduyrs, na.rm = TRUE),
              "Work: Paid work (%)" = round(mean(work == "Paid work", na.rm = TRUE) * 100, 2),
              "Work: Education (%)" = round(mean(work == "Education", na.rm = TRUE) * 100, 2),
              "Work: Retired (%)" = round(mean(work == "Retired", na.rm = TRUE) * 100, 2),
              "Work: Other (%)" = round(mean(work == "Other", na.rm = TRUE) * 100, 2),
              "Day: Monday" = round(mean(dow == "Monday", na.rm = TRUE) * 100, 2),
              "Day: Tueday" = round(mean(dow == "Tuesday", na.rm = TRUE) * 100, 2),
              "Day: Wednesday" = round(mean(dow == "Wednesday", na.rm = TRUE) * 100, 2),
              "Day: Thursday" = round(mean(dow == "Thursday", na.rm = TRUE) * 100, 2),
              "Day: Friday" = round(mean(dow == "Friday", na.rm = TRUE) * 100, 2),
              "Day: Saturday" = round(mean(dow == "Saturday", na.rm = TRUE) * 100, 2),
              "Day: Sunday" = round(mean(dow == "Sunday", na.rm = TRUE) * 100, 2),
              "Time (M)" = round(mean(tod), 2),
              "Time (SD)" = round(sd(tod), 2),
              "Quarter of year: 1st" = round(mean(quart == "1st", na.rm = TRUE) * 100, 2),
              "Quarter of year: 2nd" = round(mean(quart == "2nd", na.rm = TRUE) * 100, 2),
              "Quarter of year: 3rd" = round(mean(quart == "3rd", na.rm = TRUE) * 100, 2),
              "Quarter of year: 4th" = round(mean(quart == "4th", na.rm = TRUE) * 100, 2)) %>%
    mutate_at(vars(everything()), ~round(., digits = 2)) %>%
    pivot_longer(everything()) %>%
    print(n = 100)



#'
#' # Interviewer characteristics
#'

dat %>%
    group_by(iid) %>%
    summarise('N' = n(),
              'isex'  = mean(isex == "female"),
              "iage1" = mean(iage == "-30", na.rm = TRUE),
              "iage2" = mean(iage == "31-40", na.rm = TRUE),
              "iage3" = mean(iage == "41-50", na.rm = TRUE),
              "iage4" = mean(iage == "51-60", na.rm = TRUE),
              "iage5" = mean(iage == "61-70", na.rm = TRUE),
              "iage6" = mean(iage == "71+", na.rm = TRUE),
              "iisMo" = mean(iisMo),
              "iisFr" = mean(iisFr),
              "iisSa" = mean(iisSa),
              "iisSu" = mean(iisSu),
              .groups = "keep") %>%
    ungroup() %>%
    summarise('Number of interviewers' = n(),
              'Number of interviews (Mdn)' = median(N),
              'Number of interviews (Min)' = min(N),
              'Number of interviews (Max)' = max(N),
              'Female (%)' = round(mean(isex, na.rm = TRUE) * 100, 2),
              "Age: -30 (%)" = round(mean(iage1, na.rm = TRUE), 2),
              "Age: 31-40 (%)" = round(mean(iage2, na.rm = TRUE), 2),
              "Age: 41-50 (%)" = round(mean(iage3, na.rm = TRUE), 2),
              "Age: 51-60 (%)" = round(mean(iage4, na.rm = TRUE), 2),
              "Age: 61-70 (%)" = round(mean(iage5, na.rm = TRUE), 2),
              "Age: 71+ (%)" = round(mean(iage6, na.rm = TRUE), 2),
              "Interviews on Monday (Mdn)" = median(iisMo, na.rm = TRUE),
              "Interviews on Monday (Min)" = min(iisMo, na.rm = TRUE),
              "Interviews on Monday (Max)" = max(iisMo, na.rm = TRUE),
              "Interviews on Friday (Mdn)" = median(iisFr, na.rm = TRUE),
              "Interviews on Friday (Min)" = min(iisFr, na.rm = TRUE),
              "Interviews on Friday (Max)" = max(iisFr, na.rm = TRUE),
              "Interviews on Saturday (Mdn)" = median(iisSa, na.rm = TRUE),
              "Interviews on Saturday (Min)" = min(iisSa, na.rm = TRUE),
              "Interviews on Saturday (Max)" = max(iisSa, na.rm = TRUE),
              "Interviews on Sunday (Mdn)" = median(iisSu, na.rm = TRUE),
              "Interviews on Sunday (Min)" = min(iisSu, na.rm = TRUE),
              "Interviews on Sunday (Max)" = max(iisSu, na.rm = TRUE)) %>%
    pivot_longer(everything()) %>%
    print(n = 100)



#'
#' # Well-Being scores
#'

dat %>%
    group_by(dow) %>%
    summarise('N' = n(),
              'M (LS)' = wtd.mean(ls, wgt),
              'SD (LS)' = wtd.var(ls, wgt)^.5,
              'Mdn (LS)' = wtd.quantile(ls, wgt, .5),
              'M (HP)' = wtd.mean(happy, wgt),
              'SD (HP)' = wtd.var(happy, wgt)^.5,
              'Mdn (hp)' = wtd.quantile(happy, wgt, .5),
              .groups = "keep") %>%
    ungroup() %>%
    bind_rows(dat %>%
                  summarise('dow' = 'Week',
                            'N' = n(),
                            'M (LS)' = wtd.mean(ls, wgt),
                            'SD (LS)' = wtd.var(ls, wgt)^.5,
                            'Mdn (LS)' = wtd.quantile(ls, wgt, .5),
                            'M (HP)' = wtd.mean(happy, wgt),
                            'SD (HP)' = wtd.var(happy, wgt)^.5,
                            'Mdn (hp)' = wtd.quantile(happy, wgt, .5)))


# Correlation
cov.wt(dat[, c("ls", "happy")], wt = dat$wgt, cor = TRUE)$cor



#'
#' # Sample description
#'

#' Effect sizes for the day of week effects are calculated as standardized
#' mean differences between the mean of a specific day and the overall mean.

(sdesc <- dat %>%
            group_by(sid) %>%
            summarise('Country' = as.character(Mode(cntry)),
                      'Round' = Mode(wave),
                      'N' = n(),
                      'Female (%)' = round(mean(sex == "female") * 100, 2),
                      "Age (M)" = round(mean(age, na.rm = TRUE), 2),
                      "Age (SD)" = round(sd(age, na.rm = TRUE), 2),
                      "Paid work (%)" = round(mean(work == "Paid work", na.rm = TRUE) * 100, 2),
                      "Monday (%)" = round(mean(isMo == "Monday", na.rm = TRUE) * 100, 2),
                      "Friday (%)" = round(mean(isFr == "Friday", na.rm = TRUE) * 100, 2),
                      "Saturday (%)" = round(mean(isSa == "Saturday", na.rm = TRUE) * 100, 2),
                      "Sunday (%)" = round(mean(isSu == "Sunday", na.rm = TRUE) * 100, 2),
                      # effect sizes
                      "LS_Mo_d" = round((wtd.mean(ls, wgt) -
                                             wtd.mean(ls[isMo == "Monday"],
                                                      wgt[isMo == "Monday"])) /
                                            wtd.var(ls, wgt)^.5, 3),
                      "LS_Mo_sd_d" = round((LS_Mo_d^2 / sum(wgt))^.5, 3),
                      "LS_Fr_d" = round((wtd.mean(ls, wgt) -
                                             wtd.mean(ls[isFr == "Friday"],
                                                      wgt[isFr == "Friday"])) /
                                            wtd.var(ls, wgt)^.5, 3),
                      "LS_Fr_sd_d" = round((LS_Fr_d^2 / sum(wgt))^.5, 3),
                      "LS_Sa_d" = round((wtd.mean(ls, wgt) -
                                             wtd.mean(ls[isSa == "Saturday"],
                                                      wgt[isSa == "Saturday"])) /
                                            wtd.var(ls, wgt)^.5, 3),
                      "LS_Sa_sd_d" = round((LS_Sa_d^2 / sum(wgt))^.5, 3),
                      "LS_Su_d" = round((wtd.mean(ls, wgt) -
                                             wtd.mean(ls[isSu ==  "Sunday"],
                                                      wgt[isSu == "Sunday"])) /
                                            wtd.var(ls)^.5, 3),
                      "LS_Su_sd_d" = round((LS_Su_d^2 / sum(wgt))^.5, 3),
                      "HP_Mo_d" = round((wtd.mean(happy, wgt) -
                                             wtd.mean(happy[isMo == "Monday"],
                                                      wgt[isMo == "Monday"])) /
                                            wtd.var(happy)^.5, 3),
                      "HP_Mo_sd_d" = round((HP_Mo_d^2 / sum(wgt))^.5, 3),
                      "HP_Fr_d" = round((wtd.mean(happy, wgt) -
                                             wtd.mean(happy[isFr == "Friday"],
                                                      wgt[isFr == "Friday"])) /
                                            wtd.var(happy)^.5, 3),
                      "HP_Fr_sd_d" = round((HP_Fr_d^2 / sum(wgt))^.5, 3),
                      "HP_Sa_d" = round((wtd.mean(happy, wgt) -
                                             wtd.mean(happy[isSa == "Saturday"],
                                                      wgt[isSa == "Saturday"])) /
                                            wtd.var(happy)^.5, 3),
                      "HP_Sa_sd_d" = round((HP_Sa_d^2 / sum(wgt))^.5, 3),
                      "HP_Su_d" = round((wtd.mean(happy, wgt) -
                                             wtd.mean(happy[isSu == "Sunday"],
                                                      wgt[isSu == "Sunday"])) /
                                            wtd.var(happy)^.5, 3),
                      "HP_Su_sd_d" = round((HP_Su_d^2 / sum(wgt))^.5, 3),
                      .groups = "keep") %>%
            ungroup() %>%
            dplyr::select(-sid) %>%
            arrange(Country, Round))



#' # Save descriptives

# Format numbers
x <- sdesc %>%
        mutate(across(c('Female (%)', 'Age (M)', 'Age (SD)', 'Paid work (%)',
                        'Monday (%)', 'Friday (%)', 'Saturday (%)', 'Sunday (%)'),
                      ~format(., digits = 4))) %>%
        mutate(across(c('LS_Mo_d', 'LS_Mo_sd_d', 'LS_Fr_d', 'LS_Fr_sd_d',
                        'LS_Sa_d', 'LS_Sa_sd_d', 'LS_Su_d', 'LS_Su_sd_d',
                        'HP_Mo_d', 'HP_Mo_sd_d', 'HP_Fr_d', 'HP_Fr_sd_d',
                        'HP_Sa_d', 'HP_Sa_sd_d', 'HP_Su_d', 'HP_Su_sd_d'),
                      ~format(., digits = 5))) %>%
        mutate(across(everything(), as.character)) %>%
        mutate(across(everything(), ~ifelse(. == "0.000", "<0.001", .)))

# Save results
write.xlsx(x,
           file = here::here("Tables/Sample description.xlsx"),
           sheetName = "Samples",
           colNames = TRUE,
           rowNames = FALSE)
rm(x)
